/*****************************************************************************
 * This file is part of the Prolog Development Tools (ProDT)
 *
 * Author: Claudio Cancinos
 * WWW: https://sourceforge.net/projects/prodevtools
 * Copyright (C): 2008, Claudio Cancinos
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; If not, see <http://www.gnu.org/licenses/>
 ****************************************************************************/
package ar.com.tadp.prolog.core;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.eclipse.dltk.launching.IInterpreterInstallType;

import ar.com.tadp.prolog.core.interpreter.BPrologInstallType;
import ar.com.tadp.prolog.core.interpreter.SWIPrologInstallType;
import ar.com.tadp.prolog.core.interpreter.XSBPrologInstallType;

/**
 * @author ccancino
 *
 */
public class PrologSyntax {
	private static String swiPredFilename = "syntax" + File.separator + "swiPredicates.txt";	
	private static String swiOperFilename = "syntax" + File.separator + "swiOperators.txt";

	private static String bPredFilename = "syntax" + File.separator + "bPredicates.txt";	
	private static String bOperFilename = "syntax" + File.separator + "bOperators.txt";
	
	private static String xsbPredFilename = "syntax" + File.separator + "xsbPredicates.txt";	
	private static String xsbOperFilename = "syntax" + File.separator + "xsbOperators.txt";
	
	private static String[] swiKeywords;
	private static String[] swiOperators;
	
	private static String[] bKeywords;
	private static String[] bOperators;
	
	private static String[] xsbKeywords;
	private static String[] xsbOperators;

	
	private static String[] getDefaultKeywords() {
		if (swiKeywords == null) {
			swiKeywords = loadArray(swiPredFilename);
		}
		return swiKeywords;
	}
	
	private static String[] getDefaultOperators() {
		if (swiOperators == null) {
			swiOperators = loadArray(swiOperFilename);
		}
		return swiOperators;
	}
	
	public static String[] getKeywords(IInterpreterInstallType installType) {
		if(installType == null) {
			return getDefaultKeywords();
		}
		if (installType instanceof SWIPrologInstallType) {
			if (swiKeywords == null) {
				swiKeywords = loadArray(swiPredFilename);
			}
			return swiKeywords;
		}
		if (installType instanceof BPrologInstallType) {
			if (bKeywords == null) {
				bKeywords = loadArray(bPredFilename);
			}
			return bKeywords;
		}
		if (installType instanceof XSBPrologInstallType) {
			if (xsbKeywords == null) {
				xsbKeywords = loadArray(xsbPredFilename);
			}
			return xsbKeywords;
		}
		return getDefaultKeywords();
	}

	public static String[] getOperators(IInterpreterInstallType installType) {
		if(installType == null) {
			return getDefaultOperators();
		}
		if (installType instanceof SWIPrologInstallType) {
			if (swiOperators == null) {
				swiOperators = loadArray(swiOperFilename);
			}
			return swiOperators;
		}
		if (installType instanceof BPrologInstallType) {
			if (bOperators == null) {
				bOperators = loadArray(bOperFilename);
			}
			return bOperators;
		}
		if (installType instanceof XSBPrologInstallType) {
			if (xsbOperators == null) {
				xsbOperators = loadArray(xsbOperFilename);
			}
			return xsbOperators;
		}
		return getDefaultOperators();
	}
	
	
	private static String[] loadArray(String predFilename) {
		Vector<String>  tmp = new Vector<String>();
		loadFromFile(tmp, predFilename);
		String[] keywords = new String[tmp.size()];
		tmp.toArray(keywords);
		return keywords;
	}
	
	/**
	 * Populates the vector 'v' with the keywords inside
	 * 'filename'.
	 * 'filename' must be a relative path from the plugin
	 *  directory instalation.
	 * 
	 * @param v 'Vector' to be populated with the constants
	 *        in 'filename'.
	 * 
	 * @param filename Path and filename where we can find
	 *        the constants to be recognized.
	 * 
	 */	
	private static void loadFromFile(Vector<String> v, String filename) {
		String keyword;

		try {
			Path path = new Path(filename);
			Plugin plugin = PrologCorePlugin.getDefault();
		
			InputStream inFile = null;
			try {
				inFile = plugin.openStream(path);							
			}
			catch(IOException ioe) {
				throw (new IOException("Error opening file: " + filename + "."));
			}
			
			BufferedReader in = new BufferedReader (new InputStreamReader(inFile));						
			
			while ((keyword = in.readLine())!= null) {				
				v.add(keyword.trim());			
			}						
			in.close();
			inFile.close();
		}
		catch (IOException e) {
			logException(e);
		}
		catch (Exception e2) {
			logException(e2);	   
		}				 
	}

	private static void logException(Exception e2) {
		try {
			// Log the error to the Eclipse log.
			IStatus status = new Status(IStatus.ERROR, PrologCorePlugin.getDefault().getDescriptor()
					.getUniqueIdentifier(), 0, "Problems doing lexical analysis: " + e2.getMessage(), e2);
			// PrologCorePlugin.log(status);
		} catch (RuntimeException e) {
		}
	}
	
}
