/*****************************************************************************
 * This file is part of the Prolog Development Tools (ProDT)
 *
 * Author: Claudio Cancinos
 * WWW: https://sourceforge.net/projects/prodevtools
 * Copyright (C): 2008, Claudio Cancinos
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; If not, see <http://www.gnu.org/licenses/>
 ****************************************************************************/
package ar.com.tadp.prolog.core.builder;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.dltk.core.IScriptProject;
import org.eclipse.dltk.core.builder.IBuildContext;
import org.eclipse.dltk.core.builder.IBuildParticipant;
import org.eclipse.dltk.core.builder.IBuildParticipantExtension;
import org.eclipse.dltk.launching.ScriptRuntime;
import org.eclipse.ui.texteditor.MarkerUtilities;

import ar.com.tadp.prolog.core.PrologCorePlugin;
import ar.com.tadp.prolog.core.compiler.CompilerOutput;
import ar.com.tadp.prolog.core.compiler.CompilerOutputItem;
import ar.com.tadp.prolog.core.interpreter.PrologInstall;

/**
 * @author ccancino
 *
 */
public class PrologBuildParticipant implements IBuildParticipant, IBuildParticipantExtension {

	private final IScriptProject project;

	public PrologBuildParticipant(IScriptProject project) {
		this.project = project;
	}
	
	public boolean beginBuild(int buildType) {
		this.deleteResourceMarkers(this.getProject());
		if(buildType == IBuildContext.INCREMENTAL_BUILD) {
			this.checkOutFolders();
			// Do I need to clean the output file of the file(s) being compiled currently (??).
			return this.hasInterpreter();
		}
		if (buildType == IBuildContext.FULL_BUILD) {
			this.checkOutFolders();
			IWorkspaceRunnable op = new CleanOutFoldersOperation(this.getProject());
			try {
				if(this.hasInterpreter()) {
					ResourcesPlugin.getWorkspace().run(op, new SubProgressMonitor(new NullProgressMonitor(), 15));
				} else {
					this.cleanProjectMarkers();
					this.addLackOfInterpreterMarker();
					return false;
				}
			} catch (CoreException e) {
				PrologCorePlugin.log("Error starting compilation", e);
			}
			return true;
		}
		return false;
	}

	private boolean hasInterpreter() {
		try {
			return this.getProjectInterpreter() != null;
		} catch (CoreException e) {
			return false;
		}
	}

	public void endBuild(IProgressMonitor monitor) {
		// TODO Auto-generated method stub
	}


	public void build(IBuildContext context) throws CoreException {
		switch (context.getBuildType()) {
		case IBuildContext.RECONCILE_BUILD:
			break;
		case IBuildContext.INCREMENTAL_BUILD:
		case IBuildContext.FULL_BUILD:
			System.out.println(context.getFile());
			this.buildFile(context.getFile());
			break;
		default:
			break;
		}
		this.scheduleRefresh();
//		displayInfo(context);
	}


	private void scheduleRefresh() {
		//TODO see if I can reduce the directory scope of this method... 
		Job job = new Job("Refreshing resources...") {
			public IStatus run(final IProgressMonitor monitor) {
				IStatus result = Status.OK_STATUS;
				try {
					getProject()
							.refreshLocal(IResource.DEPTH_INFINITE, monitor);
				} catch (CoreException cex) {
					String msg = "Problem during resource refresh after build.";
					PrologCorePlugin.log(msg, cex);
					result = cex.getStatus();
				}
				return result;
			}
		};
		job.schedule();
	}

	private void checkOutFolders() {
		IWorkspaceRunnable op = new CheckOutFoldersOperation(this.getProject());
		try {
			ResourcesPlugin.getWorkspace().run(op, new NullProgressMonitor());
		} catch (CoreException cex) {
			String msg = "Problem while checking out and bin folder existence.";
			PrologCorePlugin.log(msg, cex);
		}
	}

	private void buildFile(IFile file) {
		if (this.isPrologFile(file)) {
			this.compileFile(file);
		}
	}

	private void compileFile(IFile file) {
		try {
			String relPath = file.getProjectRelativePath().makeRelative().toOSString();
			relPath = relPath.substring(relPath.indexOf(File.separator));
			String outputFile = ResourceUtil.getBinFolder(project.getProject()).getLocation().toOSString()
					+ relPath.substring(0, relPath.lastIndexOf(".")) + ".o";

			this.deleteResourceMarkers(file);
			CompilerOutput out = this.getProjectInterpreter().compile(file, outputFile, this.project);
			markResource(file, out.getErrors());
		} catch (Exception e) {
			PrologCorePlugin.log("Error compiling file: " + file.getLocation().toOSString(), e);
		}
	}

	private PrologInstall getProjectInterpreter() throws CoreException {
		return (PrologInstall) ScriptRuntime.getInterpreterInstall(this.project);
	}

	private void addLackOfInterpreterMarker() throws CoreException {
		Map marker = new HashMap();
		marker.put(IMarker.MESSAGE, "No interpreter defined for this project.");
		marker.put(IMarker.SEVERITY, Integer.valueOf(IMarker.SEVERITY_ERROR));
		marker.put(IMarker.PRIORITY, Integer.valueOf(IMarker.PRIORITY_HIGH));
		marker.put(IMarker.LOCATION, this.getProject().getFullPath().toOSString());
		MarkerUtilities.createMarker(this.getProject(), marker, IMarker.PROBLEM);
	}

	protected boolean isPrologFile(final IFile file) {
		return ResourceUtil.isPrologFile(file);
	}
	
	protected void cleanProjectMarkers() {
		try {
			this.getProject().deleteMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
		} catch (CoreException cex) {
			PrologCorePlugin.log("Could not delete markers.", cex);
		}
	}

	protected void deleteResourceMarkers(final IResource resource) {
		try {
			resource.deleteMarkers(IMarker.PROBLEM, true, IResource.DEPTH_ZERO);
		} catch (CoreException cex) {
			PrologCorePlugin.log("Could not delete markers.", cex);
		}
	}

	protected void markResource(final IFile file, final Collection<CompilerOutputItem> items) {
		try {
			for (CompilerOutputItem item : items) {
				if (file.getLocation().toString().equalsIgnoreCase(item.getFileName().trim())) {
					MarkerUtilities.createMarker(file, item.getProperties(), IMarker.PROBLEM);
				}
			}
		} catch (CoreException cex) {
			PrologCorePlugin.log("Could not create markers for compiler output.", cex);
		}
	}


	private IProject getProject() {
		return this.project.getProject();
	}

	private void displayInfo(IBuildContext context) throws CoreException {
		System.out.println("==========================================================================");
		String buildType = context.getBuildType() == IBuildContext.FULL_BUILD ? 
				"Full Build" : 
				context.getBuildType() == IBuildContext.INCREMENTAL_BUILD ? 
						"Incremental Build" : 
						"Reconcile Build";
		System.out.println("Build Type: " + buildType);
		System.out.println("File Name: " + context.getFile().getName());
		System.out.println("File Content: " + new String(context.getContents()));
		System.out.println("Referenced: " + this.getProjectReferences(this.project.getProject().getReferencedProjects()));
		System.out.println("Referencing: " + this.getProjectReferences(this.project.getProject().getReferencingProjects()));
	}

	private String getProjectReferences(IProject[] projects) {
		List list = new ArrayList();
		for (int i = 0; i < projects.length; i++) {
			list.add(projects[i]);
		}
		return list.toString();
	}

}
