/*****************************************************************************
 * This file is part of the Prolog Development Tools (ProDT)
 *
 * Author: Claudio Cancinos
 * WWW: https://sourceforge.net/projects/prodevtools
 * Copyright (C): 2008, Claudio Cancinos
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; If not, see <http://www.gnu.org/licenses/>
 ****************************************************************************/
package ar.com.tadp.prolog.core.builder;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Arrays;
import java.util.List;

import org.eclipse.dltk.launching.IScriptProcessHandler;

import ar.com.tadp.prolog.core.ThreadExecutorService;

/**
 * 
 * @author ccancino
 */
public class PrologProcessHandler implements IScriptProcessHandler {
    private final Object lock = new Object();
	private Process process;
	private ScriptOutputReader stdoutReader;
	private ScriptOutputReader stderrReader;

    public ScriptResult handle(Process process, char[] stdin) {
        this.process = process;
		stdoutReader = new ScriptOutputReader(process.getInputStream());
        stderrReader = new ScriptOutputReader(process.getErrorStream());
        
        try {
        	ThreadExecutorService.getInstance().execute(stdoutReader);
        	ThreadExecutorService.getInstance().execute(stderrReader);

            writeToStdin(process.getOutputStream(), stdin);

            synchronized (lock) {
                try {
                    lock.wait();
                    // ???
                    process.waitFor();
                }
                catch (InterruptedException e) {   
                    // do nothing
                }
            }
        }
        finally {
            // make sure the process is terminated!
            process.destroy();
        }

        return buildResult(process, stdoutReader, stderrReader);
    }

    private ScriptResult buildResult(Process process, ScriptOutputReader stdoutReader, ScriptOutputReader stderrReader) {
        ScriptResult result = new ScriptResult();
        result.exitValue = process.exitValue();

        result.stdout = stdoutReader.getBuffer();
        result.stderr = stderrReader.getBuffer();

        result.stdoutLines = stdoutReader.getLines();
        result.stderrLines = stderrReader.getLines();
        return result;
    }

    private void closeWriter(OutputStreamWriter writer) {
        if (writer != null) {
            try {
                writer.close();
            }
            catch (IOException e) {
                // do nothing
            }
        }
    }

    private void writeToStdin(OutputStream stream, char[] stdin) {
        OutputStreamWriter stdinWriter = new OutputStreamWriter(stream);

        try {
            if (stdin != null) {
                stdinWriter.write(stdin);
                stdinWriter.flush();
            }
        }
        catch (IOException e) {
            // broken pipe, ignore
        }
        finally {
            closeWriter(stdinWriter);
        }   
    }
    
    public void kill() {
    	this.stderrReader.kill();
    	this.stdoutReader.kill();
    	this.process.destroy();
    }
    
    class ScriptOutputReader implements Runnable {
        private StringBuffer buffer = new StringBuffer();
        private InputStream stream;
		private boolean stop = false;

		ScriptOutputReader(InputStream stream) {
			this.stream = stream;
		}

        public void kill() {
        	this.stop = true;
		}

		public void run() {
            BufferedReader reader = new BufferedReader(new InputStreamReader(stream));

            try {
                int count = 0;
                char[] bytes = new char[1024];
                String previous = "";
                String current;

				while (!this.stop && (count = reader.read(bytes)) >= 0) {
					current = new String(bytes);
					this.verifyLooping(previous, current);
					previous = current;
					buffer.append(bytes, 0, count);
				}

				synchronized (lock) {
					lock.notifyAll();
				}
            } catch (IOException e) {
            	buffer.setLength(0);
			}
        }
		
		/**
		 * This is here in order to avoid being trapped in compiler's infinite loop
		 * @param previous
		 * @param current
		 */
		private void verifyLooping(String previous, String current) {
			if (previous.equals(current)) {
				PrologProcessHandler.this.kill();
			}
		}

		public String getBuffer() {
			return buffer.toString();
		}

		public List<String> getLines() {
			String[] lines = buffer.toString().split("\n");
			return Arrays.asList(lines);
		}
    }

}
