/*****************************************************************************
 * This file is part of the Prolog Development Tools (ProDT)
 *
 * Author: Claudio Cancinos
 * WWW: https://sourceforge.net/projects/prodevtools
 * Copyright (C): 2008, Claudio Cancinos
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; If not, see <http://www.gnu.org/licenses/>
 ****************************************************************************/
package ar.com.tadp.prolog.core.compiler;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.CoreException;

/**
 * <p>
 * Default implementation for the information that was parsed out of the
 * compiler output.
 * </p>
 * 
 * @author Claudio
 */
public class CompilerOutputItem {
	private String fileName;
	private int lineNumber;
	private String comment;
	private int startColumn;
	private int endColumn;
	private int severity;

	public CompilerOutputItem() {
		// placeholder constructor
	}

	public CompilerOutputItem(int severity) {
		this.severity = severity;
	}

	public CompilerOutputItem(final String fileName, final int lineNumber, final String comment) {
		this.fileName = fileName;
		this.lineNumber = lineNumber;
		this.comment = comment.trim();
	}

	public CompilerOutputItem(String fileName, int lineNumber, int startColumn, int endColumn, String comment) {
		this.fileName = fileName;
		this.lineNumber = lineNumber;
		this.startColumn = startColumn;
		this.endColumn = endColumn;
		this.comment = comment;
	}

	public void addToComment(final String commentAddition) {
		String start = (comment.equals("")) ? comment : comment + "\n";
		this.comment = start + commentAddition.trim();
	}

	public String toString() {
		return "CompilerOutputItem:" + "\n  file   : " + fileName
				+ "\n  line   : " + lineNumber + "\n  comment: " + comment;
	}

	public int getLine() {
		return lineNumber;
	}

	public int getStartColumn() {
		return startColumn;
	}

	public int getEndColumn() {
		return endColumn;
	}

	public String getComment() {
		return comment;
	}

	public String getFileName() {
		return fileName;
	}

	public Map getProperties() {
		Map marker = new HashMap();
		marker.put(IMarker.MESSAGE, comment);
		marker.put(IMarker.SEVERITY, Integer.valueOf(this.severity));
		marker.put(IMarker.PRIORITY, Integer.valueOf(IMarker.PRIORITY_HIGH));
		marker.put(IMarker.LINE_NUMBER, Integer.valueOf(lineNumber));
		marker.put(IMarker.CHAR_START, Integer.valueOf(startColumn));
		marker.put(IMarker.CHAR_END, Integer.valueOf(startColumn + endColumn));
		return marker;
	}
	
	public void populateMarker(IMarker marker) throws CoreException {
		marker.setAttribute(IMarker.MESSAGE, comment);
		marker.setAttribute(IMarker.SEVERITY, this.severity);
		marker.setAttribute(IMarker.PRIORITY, Integer.valueOf(IMarker.PRIORITY_HIGH));
		marker.setAttribute(IMarker.LINE_NUMBER, Integer.valueOf(lineNumber));
		if (this.endColumn != 0) {
			marker.setAttribute(IMarker.CHAR_START, Integer.valueOf(startColumn));
			marker.setAttribute(IMarker.CHAR_END, Integer.valueOf(startColumn + endColumn));
		}
	}

	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	public void setLine(int line) {
		this.lineNumber = line;
	}

	public void setStartColumn(int column) {
		this.startColumn = column;
	}

	public void setEndColumn(int column) {
		this.endColumn = column;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}

}