/*****************************************************************************
 * This file is part of the Prolog Development Tools (ProDT)
 *
 * Author: Claudio Cancinos
 * WWW: https://sourceforge.net/projects/prodevtools
 * Copyright (C): 2008, Claudio Cancinos
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; If not, see <http://www.gnu.org/licenses/>
 ****************************************************************************/
package ar.com.tadp.prolog.core.compiler.errorparsers;

import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;

import ar.com.tadp.prolog.core.PrologCorePlugin;
import ar.com.tadp.prolog.core.builder.ResourceUtil;
import ar.com.tadp.prolog.core.compiler.CompilerOutput;
import ar.com.tadp.prolog.core.compiler.CompilerOutputItem;

/**
 * @author ccancino
 *
 */
public class BCompilerErrorsParser extends CompilerErrorsParser {
	private final static Pattern SYNTAX_ERROR = Pattern.compile("^\\*+ syntax error \\*+$"); 
	private final static Pattern WARNING = Pattern.compile("^\\*+ warning \\*+(.+):\\((.+),(.+)-(.+)\\)::(.+)$");
	
	public void doParse(IFile file, String outputString, CompilerOutput output) {
		StringTokenizer tn = new StringTokenizer(outputString, ResourceUtil.LINE_SEPARATOR);;
        while(tn.hasMoreTokens()) {
        	String line = tn.nextToken();
        	
			try {
				Matcher m = SYNTAX_ERROR.matcher(line);
				if(m.matches()){
					this.addError(output, tn.nextToken(), file);
				}
				
				m = WARNING.matcher(line);
				if(m.matches()){
					this.addWarning(output, m);
				}
			} catch (Exception e) {
				PrologCorePlugin.log("Error trying to parse error line: " + line, e);
			}
		}
	}

	private void addWarning(CompilerOutput output, Matcher m) {
		CompilerOutputItem item = new CompilerOutputItem(IMarker.SEVERITY_WARNING);
		item.setFileName(m.group(2));
		item.setLine(Integer.parseInt(m.group(3)));
		item.setStartColumn(this.getOffset(item.getLine()));
		item.setEndColumn(this.getLength(item.getLine()));
		item.setComment(m.group(1) + " " + m.group(5));
		output.addError(item);
	}
	
	private void addError(CompilerOutput output, String line, IFile file) {
		int startOfHere = line.indexOf("<<");
		if (startOfHere != -1) {
			String head = line.substring(0, startOfHere);
			String tail = line.substring(line.lastIndexOf(">>") + 2, line.length());
			int errorLine = this.getLine(file, head + tail);
			errorLine = (errorLine == -1) ? this.getLine(file, head) : errorLine;
			CompilerOutputItem item = new CompilerOutputItem(IMarker.SEVERITY_ERROR);
			item.setFileName(file.getLocation().toPortableString());
			item.setLine(errorLine);
			item.setStartColumn(this.getOffset(item.getLine()));
			item.setEndColumn(this.getLength(item.getLine()));
			item.setComment(line);
			output.addError(item);
		}
	}
	
	@Override
	protected int getLine(IFile file, String pattern) {
		int line = super.getLine(file, pattern);
		if(line == -1) {
			pattern = pattern.replaceAll(" ", "");
			int lineIndex = 0;
			for (String fileLine : this.fileLines) {
				lineIndex++;
				fileLine  = fileLine.replaceAll(" ", "");
				if(pattern.startsWith(fileLine)) {
					return lineIndex;
				}
			}
			return -1;
			
		}
		return line;
	}

}
