/*****************************************************************************
 * This file is part of the Prolog Development Tools (ProDT)
 *
 * Author: Claudio Cancinos
 * WWW: https://sourceforge.net/projects/prodevtools
 * Copyright (C): 2008, Claudio Cancinos
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; If not, see <http://www.gnu.org/licenses/>
 ****************************************************************************/
package ar.com.tadp.prolog.core.completion;

import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.dltk.codeassist.ICompletionEngine;
import org.eclipse.dltk.compiler.env.IModuleSource;
import org.eclipse.dltk.core.CompletionProposal;
import org.eclipse.dltk.core.CompletionRequestor;
import org.eclipse.dltk.core.DLTKCore;
import org.eclipse.dltk.core.IField;
import org.eclipse.dltk.core.IMethod;
import org.eclipse.dltk.core.IModelElement;
import org.eclipse.dltk.core.IModelElementVisitor;
import org.eclipse.dltk.core.IScriptProject;
import org.eclipse.dltk.core.IType;
import org.eclipse.dltk.core.ModelException;

import ar.com.tadp.prolog.core.PrologCorePlugin;
import ar.com.tadp.prolog.core.PrologSyntax;

/**
 * @author ccancino
 *
 */
public class PrologCompletionEngine implements ICompletionEngine {
	private IScriptProject project;
	private CompletionRequestor requestor;
	private int actualCompletionPosition;
	private int offset;


	public void setProgressMonitor(IProgressMonitor progressMonitor) {
		// TODO Auto-generated method stub
		
	}
 
	public void complete(IModuleSource module, int position, int pos) {
		this.actualCompletionPosition = position;
		this.offset = pos;
		String[] keywords = PrologSyntax.getKeywords(PrologCorePlugin.getDefaultInterpreter(module.getModelElement().getScriptProject()).getInterpreterInstallType());
		
		final String prefix = this.getPrefix(module.getSourceContents(), this.actualCompletionPosition);
		this.actualCompletionPosition -= prefix.length();
		
		for (int j = 0; j < keywords.length; j++) {
			if(keywords[j].startsWith(prefix)) {
				createProposal(keywords[j], null);
			}
		}
 
		// Completion for model elements.
		try {
			module.getModelElement().accept(new IModelElementVisitor() {
				public boolean visit(IModelElement element) {
					if (element.getElementType() > IModelElement.SOURCE_MODULE) {
						if (element.getElementName().startsWith(prefix)) {
							createProposal(element.getElementName(), element);
						}
					}
					return true;
				}
			});
		} catch (ModelException e) {
			if (DLTKCore.DEBUG) {
				e.printStackTrace();
			}
		}
	}

	private void createProposal(String name, IModelElement element) {
		CompletionProposal proposal = null;
		try {
			if (element == null) {
				proposal = this.createProposal(CompletionProposal.KEYWORD,
						this.actualCompletionPosition);
			} else {
				switch (element.getElementType()) {
				case IModelElement.METHOD:
					proposal = this.createProposal(
							CompletionProposal.METHOD_DECLARATION,
							this.actualCompletionPosition);
					proposal.setFlags(((IMethod) element).getFlags());
					break;
				case IModelElement.FIELD:
					proposal = this.createProposal(
							CompletionProposal.FIELD_REF,
							this.actualCompletionPosition);
					proposal.setFlags(((IField) element).getFlags());
					break;
				case IModelElement.TYPE:
					proposal = this.createProposal(CompletionProposal.TYPE_REF,
							this.actualCompletionPosition);
					proposal.setFlags(((IType) element).getFlags());
					break;
				default:
					proposal = this.createProposal(CompletionProposal.KEYWORD,
							this.actualCompletionPosition);
					break;
				}
			}
			proposal.setName(name);
			proposal.setCompletion(name);
			proposal.setReplaceRange(actualCompletionPosition - offset,
					actualCompletionPosition - offset);
			proposal.setRelevance(20);
			proposal.setModelElement(element);
			this.requestor.accept(proposal);
		} catch (Exception e) {
		}
	}
	
	public void setOptions(Map options) {
	}
	
	public void setProject(IScriptProject project) {
		this.project = project;
	}
	
	public void setRequestor(CompletionRequestor requestor) {
		this.requestor = requestor;
	}
	
	protected CompletionProposal createProposal(int kind, int completionOffset) {
		return CompletionProposal.create(kind,
				completionOffset - this.offset);
	}
	
	private String getPrefix(String sourceContents, int currentOffset) {
		int i= currentOffset;
		if (i > sourceContents.length()) {
			return "";
		}
		
		while (i > 0) {
			char ch= sourceContents.substring(i - 1, i).charAt(0);
			if (isInvalidCharacter(ch)) {
				break;
			}
			i--;
		}
		return sourceContents.substring(i, currentOffset).trim();
	}

	private boolean isInvalidCharacter(char ch) {
		return ch != '\r' && ch != '\n' && !Character.isJavaIdentifierPart(ch) && ch != '(' && ch != ')';
	}

}
