/*****************************************************************************
 * This file is part of the Prolog Development Tools (ProDT)
 *
 * Author: Claudio Cancinos
 * WWW: https://sourceforge.net/projects/prodevtools
 * Copyright (C): 2008, Claudio Cancinos
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; If not, see <http://www.gnu.org/licenses/>
 ****************************************************************************/
package ar.com.tadp.prolog.core.ui.editor.text;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.dltk.core.IScriptProject;
import org.eclipse.dltk.launching.IInterpreterInstallType;
import org.eclipse.dltk.ui.text.AbstractScriptScanner;
import org.eclipse.dltk.ui.text.IColorManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.PlatformUI;

import ar.com.tadp.prolog.core.PrologCorePlugin;
import ar.com.tadp.prolog.core.PrologSyntax;
import ar.com.tadp.prolog.core.builder.ResourceUtil;
import ar.com.tadp.prolog.core.ui.editor.PrologPartitionScanner;
import ar.com.tadp.prolog.core.ui.editor.PrologSourceViewerConfiguration;
import ar.com.tadp.prolog.core.ui.editor.PrologVarDetector;
import ar.com.tadp.prolog.core.ui.editor.text.detectors.ListWordsRule;
import ar.com.tadp.prolog.core.ui.editor.text.detectors.PrologWhitespaceDetector;
import ar.com.tadp.prolog.core.ui.editor.text.detectors.PrologWordDetector;

/**
 * This scanner will be executed on a single partition of the document
 * (partitions are defined by {@link PrologPartitionScanner}) Inside this
 * partition, it will define which color will be used for each word based on
 * rules defining coloring area usually by defining starting and ending points.
 * 
 * @see {@link PrologSourceViewerConfiguration}, {@link PrologPartitionScanner}
 * @author ccancino
 */
public class PrologCodeScanner extends AbstractScriptScanner {
	
	private static String tokenProperties[] = new String[] {
			PrologColorConstants.PROLOG_DEFAULT,
			PrologColorConstants.PROLOG_KEYWORD, 
			PrologColorConstants.PROLOG_OPERATOR,
			PrologColorConstants.PROLOG_VARIABLE,
			PrologColorConstants.PROLOG_STRING,
			PrologColorConstants.PROLOG_PREDICATE_DECLARATION};
	
	public PrologCodeScanner(IColorManager manager,	IPreferenceStore store) {
		super(manager, store);
		this.initialize();
	}

	protected String[] getTokenProperties() {
		return tokenProperties;
	}

	protected List createRules() {
		List rules = new ArrayList();
		IToken operator = this.getToken(PrologColorConstants.PROLOG_OPERATOR);
		IToken keyword = this.getToken(PrologColorConstants.PROLOG_KEYWORD);
		IToken predicateDeclaration = this.getToken(PrologColorConstants.PROLOG_PREDICATE_DECLARATION);
		IToken variable = this.getToken(PrologColorConstants.PROLOG_VARIABLE);
		IToken string = this.getToken(PrologColorConstants.PROLOG_STRING);
		IToken other = this.getToken(PrologColorConstants.PROLOG_DEFAULT);
		
		// Add generic whitespace rule.
		rules.add(new WhitespaceRule(new PrologWhitespaceDetector()));

		// Add a rule for quotes
		rules.add(new SingleLineRule("\"", "\"", string, '\\'));
		rules.add(new SingleLineRule("'", "'", string, '\\'));

		// Rule for variables.
		rules.add(new WordRule(new PrologVarDetector(), variable));
		
		// Rule to detect predicate definitions.
		rules.add(new PredicateDefinitionPatternRule(new PrologWordDetector(), ResourceUtil.LINE_SEPARATOR, "(", predicateDeclaration));
		
		// Add word rule for Operators.
		rules.add(new ListWordsRule(new PrologOperatorDetector(), PrologSyntax.getOperators(this.getInterpreter()), operator, other));
		
		// Add word rule for keywords.
		rules.add(new ListWordsRule(new PrologWordDetector(), PrologSyntax.getKeywords(this.getInterpreter()), keyword, other));

		this.setDefaultReturnToken(other);
		
		return rules;
	}

	private IInterpreterInstallType getInterpreter() {
		try {
			ISelectionService selectionService = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getSelectionService();
			ISelection selection = selectionService.getSelection();
			
			TreePath treePath = ((TreeSelection)selection).getPaths()[0];
			IScriptProject segment = (IScriptProject) treePath.getSegment(0);
			return PrologCorePlugin.getDefaultInterpreter(segment).getInterpreterInstallType();
		} catch (Exception e) {
			return null;
		}
	}

}
